@@ -56,3 +56,38 @@ xh_param_assign_pattern(xh_pattern_t *patt, SV *value)
         patt->enable = FALSE;
     }
 }
+
+void
+xh_param_assign_filter(xh_pattern_t *patt, SV *value)
+{
+    if (patt->expr != NULL) {
+        SvREFCNT_dec(patt->expr);
+        patt->expr = NULL;
+    }
+
+    if ( SvOK(value) ) {
+        patt->enable = TRUE;
+        patt->always = FALSE;
+        patt->expr   = value;
+        SvREFCNT_inc(value);
+    }
+    else {
+        patt->enable = FALSE;
+    }
+}
+
+SV *
+xh_param_assign_cb(char *name, SV *value)
+{
+    if ( !SvOK(value) )
+        return NULL;
+
+    if ( !SvROK(value) || SvTYPE(SvRV(value)) != SVt_PVCV)
+        croak("Parameter '%s' is not CODE reference", name);
+
+    value = SvRV(value);
+
+    SvREFCNT_inc(value);
+
+    return value;
+}
