@@ -97,6 +97,24 @@
     XH_X2H_PROCESS_STATE(NODE_PARSE_ATTR_VALUE_START)                   \
     XH_X2H_PROCESS_STATE(NODE_1_END_ATTR_VALUE_START)                   \
     XH_X2H_PROCESS_STATE(NODE_2_END_ATTR_VALUE_START)                   \
+    XH_X2H_PROCESS_STATE(STYLESHEET_PI_1_1_1_START)                     \
+    XH_X2H_PROCESS_STATE(STYLESHEET_PI_1_1_2_START)                     \
+    XH_X2H_PROCESS_STATE(STYLESHEET_PI_1_1_3_START)                     \
+    XH_X2H_PROCESS_STATE(STYLESHEET_PI_1_2_1_START)                     \
+    XH_X2H_PROCESS_STATE(STYLESHEET_PI_1_2_2_START)                     \
+    XH_X2H_PROCESS_STATE(STYLESHEET_PI_2_1_1_START)                     \
+    XH_X2H_PROCESS_STATE(STYLESHEET_PI_2_1_2_START)                     \
+    XH_X2H_PROCESS_STATE(STYLESHEET_PI_2_1_3_START)                     \
+    XH_X2H_PROCESS_STATE(STYLESHEET_PI_2_2_1_START)                     \
+    XH_X2H_PROCESS_STATE(STYLESHEET_PI_2_2_2_START)                     \
+    XH_X2H_PROCESS_STATE(STYLESHEET_PI_ATTR_START)                      \
+    XH_X2H_PROCESS_STATE(STYLESHEET_PI_ATTR_SEARCH_ATTR_START)          \
+    XH_X2H_PROCESS_STATE(PI_SEARCH_END_TAG2_START)                      \
+    XH_X2H_PROCESS_STATE(STYLESHEET_PI_ATTR_PARSE_ATTR_NAME_START)      \
+    XH_X2H_PROCESS_STATE(STYLESHEET_PI_ATTR_ATTR_SKIP_BLANK_START)      \
+    XH_X2H_PROCESS_STATE(STYLESHEET_PI_ATTR_PARSE_ATTR_VALUE_START)     \
+    XH_X2H_PROCESS_STATE(STYLESHEET_PI_ATTR_1_END_ATTR_VALUE_START)     \
+    XH_X2H_PROCESS_STATE(STYLESHEET_PI_ATTR_2_END_ATTR_VALUE_START)     \
     XH_X2H_PROCESS_STATE(XML_DECL_1_START)                              \
     XH_X2H_PROCESS_STATE(XML_DECL_2_START)                              \
     XH_X2H_PROCESS_STATE(XML_DECL_3_START)                              \
